=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-07-24 03:34:16 +0000
+++ bzrlib/tests/__init__.py	2006-07-24 06:54:03 +0000
@@ -1314,6 +1314,7 @@
                    'bzrlib.tests.test_transactions',
                    'bzrlib.tests.test_transform',
                    'bzrlib.tests.test_transport',
+                   'bzrlib.tests.test_tree',
                    'bzrlib.tests.test_tsort',
                    'bzrlib.tests.test_tuned_gzip',
                    'bzrlib.tests.test_ui',

=== added file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_tree.py	2006-07-24 06:54:03 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for Tree and InterTree."""
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tree import InterTree
+
+
+class TestInterTree(TestCaseWithTransport):
+
+    def test_revision_tree_revision_tree(self):
+        # we should have an InterTree registered for RevisionTree to
+        # RevisionTree.
+        tree = self.make_branch_and_tree('.')
+        rev_id = tree.commit('first post')
+        rev_id2 = tree.commit('second post', allow_pointless=True)
+        rev_tree = tree.branch.repository.revision_tree(rev_id)
+        rev_tree2 = tree.branch.repository.revision_tree(rev_id2)
+        optimiser = InterTree.get(rev_tree, rev_tree2)
+        self.assertIsInstance(optimiser, InterTree)
+        optimiser = InterTree.get(rev_tree2, rev_tree)
+        self.assertIsInstance(optimiser, InterTree)
+
+    def test_working_tree_revision_tree(self):
+        # we should have an InterTree available for WorkingTree to 
+        # RevisionTree.
+        tree = self.make_branch_and_tree('.')
+        rev_id = tree.commit('first post')
+        rev_tree = tree.branch.repository.revision_tree(rev_id)
+        optimiser = InterTree.get(rev_tree, tree)
+        self.assertIsInstance(optimiser, InterTree)
+        optimiser = InterTree.get(tree, rev_tree)
+        self.assertIsInstance(optimiser, InterTree)
+
+    def test_working_tree_working_tree(self):
+        # we should have an InterTree available for WorkingTree to 
+        # WorkingTree.
+        tree = self.make_branch_and_tree('1')
+        tree2 = self.make_branch_and_tree('2')
+        optimiser = InterTree.get(tree, tree2)
+        self.assertIsInstance(optimiser, InterTree)
+        optimiser = InterTree.get(tree2, tree)
+        self.assertIsInstance(optimiser, InterTree)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-07-24 02:05:29 +0000
+++ bzrlib/tree.py	2006-07-24 06:54:03 +0000
@@ -25,6 +25,7 @@
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib import errors
 from bzrlib.inventory import Inventory
+from bzrlib.inter import InterObject
 from bzrlib.osutils import fingerprint_file
 import bzrlib.revision
 from bzrlib.trace import mutter, note
@@ -320,3 +321,20 @@
         interesting_ids.update(new_pending)
         pending = new_pending
     return interesting_ids
+
+
+class InterTree(InterObject):
+    """This class represents operations taking place between two Trees.
+
+    Its instances have methods like 'compare' and contain references to the
+    source and target trees these operations are to be carried out on.
+
+    clients of bzrlib should not need to use InterTree directly, rather they
+    should use the convenience methods on Tree such as 'Tree.compare()' which
+    will pass through to InterTree as appropriate.
+    """
+
+    _optimisers = set()
+
+
+

